/* attribute.h -*- C++ -*- attribute interface
 * Rob Earhart
 * $Id: attribute.h,v 1.2 1998/02/18 21:16:38 rob Exp $
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_ATTRIBUTE_H
#define ACAP_ATTRIBUTE_H

#include "dsobjects.h"
#include "dstring.h"
#include "str.h"

// XXX Accessor functions should check for data consistancy, and do
// the Right Thing (hand back something reasonable, throw an
// exception, alert the user and hand back something reasonable, ...)

class Dataset;

class Attribute {
public:
  // Reading
  typedef vector<DString> values_t;
  typedef vector<DString> acls_t;

  Attribute(Dataset *_dataset, attrval_t *_data)
    : dataset(_dataset), data(_data) {}
  Attribute(const Attribute &it) : dataset(it.dataset), data(it.data) {}
  Attribute &operator=(const Attribute &it) {
    dataset = it.dataset;
    data = it.data;
    return *this;
  }
  DString Name() const {
    return DString(&data->name);
  }
  bool Exist() const {
    return data ? true : false;
  }
  bool IsValued() const {
    return data && (data->state == attrval_t::single
		    || data->state == attrval_t::multi);
  }
  bool InDate(Timestamp::Stamp stamp) const {
    return data->in_date(stamp);
  }
  bool IsNil() const {
    return data->state == attrval_t::nil;
  }
  bool IsDefault() const {
    return data->state == attrval_t::def;
  }
  bool IsString() const {
    return data && data->state == attrval_t::single;
  }
  bool IsMulti() const {
    return data && data->state == attrval_t::multi;
  }
  DString StringValue() const {
    return DString(data->values->buckets(0));
  }
  size_t NValues() const {
    return data->values->n_alloced;
  }
  void MultiValue(values_t &values) const {
    values.reserve(NValues());
    for (unsigned lupe = 0; lupe < NValues(); lupe++)
      values.push_back(DString(data->values->buckets(lupe)));
  }
  bool HaveContentType() const {
    return data->havectype;
  }
  DString ContentType() const {
    return DString(&data->contenttype);
  }
  bool HaveACL() const {
    return data->acls != NULL;
  }
  size_t NACLs() const {
    return data->acls->n_alloced;
  }
  void ACLValue(acls_t acls) const {
    acls.reserve(NACLs());
    for (unsigned lupe = 0; lupe < NACLs(); lupe++)
      acls.push_back(DString(data->acls->buckets(lupe)));
  }
  Timestamp::Stamp CreateTime() const {
    return data->createtime;
  }

  // Writing
  void SetDelete(Trx *trx) {
    data->set_delete(trx->TrxStamp(), trx);
  }
  void SetContentType(Trx *trx,
		      const String &ctype);
  void SetNilValue(Trx *trx) {
    data->set_state_nil(trx);
  }
  void SetDefaultValue(Trx *trx) {
    data->set_state_def(trx);
  }
  void SetSingleValue(Trx *trx,
		      const String &value);
  typedef vector<String> setvalues_t;
  void SetMultiValue(Trx *trx,
		     const setvalues_t &values);
private:
  Dataset *dataset;
  attrval_t *data;
};

#endif // ACAP_ATTRIBUTE_H
