/* context.cc - context implementation
 * Rob Earhart
 * $Id: context.cc,v 1.7 1998/03/09 00:58:01 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include "context.h"
#include "dataset.h"
#include "conn.h"
#include "sort.h"
#include "dsgroup.h"
#include "entryref.h"

Context::Context(Conn *_conn,
		 const Timestamp::Stamp &_modtime,
		 bool _enumerated)
  : conn(_conn), modtime(_modtime), enumerated(_enumerated) {
    conn->Ref();
}

Context::~Context() {
  conn->Unref();
}

void Context::Search(Context::entries_t &result,
		     DatasetGroup *group,
		     NotifyContext *context,
		     const String &path,
		     Modifiers *modifiers,
		     Criteria *criteria,
		     Referer *referer,
		     Timestamp::Stamp stamp) {
  Enumerate(result, group, stamp);

  // Filter...
  if (criteria)
    result.erase(remove_if(result.begin(), result.end(), CriteriaPtr(criteria)),
                  result.end());

  // Sort...
  // We do the weird casting here because we *don't* want
  // to be using Ptr<EntryRef>; introsort's partition step
  // copies out a value, which leaves a NULL behind, which
  // causes us to lose.
  if (modifiers && modifiers->GotSort())
    sort((vector<EntryRef *>::iterator)result.begin(),
	 (vector<EntryRef *>::iterator)result.end(),
	 Sort(modifiers->SortList()));

  // Update positions...
  for (unsigned ent_number = 0;
       ent_number < result.size();
       ent_number++)
    result[ent_number]->SetEntryNumber(ent_number + 1);

  // Done.
}

void Context::DoneInitializing(Context::entries_t &ents,
			       Ptr<DatasetGroup> &group,
			       Ptr<Criteria> &crit,
			       Ptr<Modifiers> &mods) {
  dsgroup = group;
  entries.swap(ents);
  state = good;
  dsgroup->Close();
}
