/* dsgroup.h -*- C++ -*- dataset group interface
 * Rob Earhart
 * $Id: dsgroup.h,v 1.2 1998/03/09 00:58:13 rob Exp $
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_DSGROUP_H
#define ACAP_DSGROUP_H

#include <hash_set.h>
#include "refcount.h"
#include "void.h"

class Dataset;
class PSBlob;
class DatasetGroup;
class PSResources;

class DatasetGroup {
public:
  void AddDataset(Dataset *dataset);
  void AddBlob(PSBlob *blob);
  void AddGroup(DatasetGroup *group);
  void Open();			// opens all datasets in group
  void Close();			// closes all datasets in group
private:
  typedef hash_set<P<Dataset>, hash<void *>, equal_to<void *> > datasets_t;
  datasets_t datasets;
  typedef hash_set<P<PSBlob>, hash<void *>, equal_to<void *> > blobs_t;
  blobs_t blobs;
  typedef hash_set<P<PSResources>, hash<void *>, equal_to<void *> > data_t;
  data_t data;
};

class DatasetGroupGrab {
public:
  DatasetGroupGrab(DatasetGroup *_group) : group(_group) {
    group->Open();
  }
  ~DatasetGroupGrab() {
    group->Close();
  }
private:
  DatasetGroup *group;
};

#endif // ACAP_DSGROUP_H
