#!/usr/local/bin/perl5

die "Usage: $0 dataset\n" unless (@ARGV == 1);

open(D, $ARGV[0]) || die "Unable to open $ARGV[0]: $!\n";

seek(D, 0x1004, 0) || die "seek: $!\n"; # first page + page header
&extract_dataset();

sub indent {
    local($,) = ".";
    print @indent, "";
}

sub extract_base {
    &indent;
    print("base: ", &extract_offset(), "\n");
}

sub extract_offset {
    local($data);
    read(D, $data, 4);
    die "read: $!\n" if $!;
    return unpack("L", $data);
}

sub extract_timestamp {
    local($name) = @_;
    &indent;
    print "$name: ", &extract_offset, " seconds + ", &extract_offset, "\n";
}

sub extract_hex {
    local($name) = @_;
    &indent;
    printf("$name: 0x%lx\n", &extract_offset);
}

sub extract_integer {
    local($name) = @_;
    local($result) = &extract_offset;
    &indent;
    printf("$name: $result\n");
    return $result;
}

sub extract_attrval {
    &extract_stringval("name");
    &extract_timestamp("createtime");
    &extract_timestamp("deletetime");
    &indent;
    print "state: ", ("nil", "def", "single", "multi")[&extract_offset], "\n";
    &extract_boolean("havectype");
    &extract_list("values", \&extract_stringval);
    &extract_stringval("contenttype");
    &extract_list("acls", \&extract_stringval);
}

sub extract_stringval {
    local($name) = @_;
    push(@indent, $name);
    &extract_hex("hash");
    local($offset) = &extract_offset;
    &indent;
    printf("value at 0x%lx: ", $offset);
    if ($offset) {
	local($original) = tell(D);
	seek(D, $offset, 0) || die "seek: $!\n";
	&extract_offset;	# base
	&extract_offset;	# next;
	local($size) = &extract_offset;
	local($data);
	read(D, $data, $size);
	die "read: $!\n" if $!;
	print("$size bytes => \"", $data, "\"\n");
	seek(D, $original, 0) || die "seek: $!\n";
    } else {
	print "NIL\n";
    }
    pop(@indent);
}

sub extract_boolean {
    local($name) = @_;
    &indent;
    if (&extract_offset) {
	print "$name: true\n";
    } else {
	print "$name: false\n";
    }
}

sub extract_entry {
    &extract_stringval("name");
    &extract_timestamp("createtime");
    &extract_timestamp("modtime");
    &extract_timestamp("deletetime");
    &extract_boolean("inherit");
    &extract_boolean("freeattrs");
    &extract_list("attrs", \&extract_attrval);
}

sub extract_index {
    &indent;
    print "(index data)\n";
}

sub extract_list {
    local($name, $extractor) = @_;
    local($offset) = &extract_offset();
    &indent;
    printf("$name at 0x%lx\n", $offset);
    if ($offset) {
	push(@indent, $name);
	local($original) = tell(D);
	seek(D, $offset, 0) || die "seek: $!\n";
	&extract_base();
	&extract_hex("next");
	&extract_integer("n_buckets");
	local($n_alloced) = &extract_integer("n_alloced");
	local($lupe);
	for ($lupe = 0; $lupe < $n_alloced; $lupe++) {
	    &indent;
	    printf("$lupe is at offset 0x%lx\n", tell(D));
	    push(@indent, $lupe);
	    &$extractor;
	    pop(@indent);
	}
	pop(@indent);
	seek(D, $original, 0) || die "seek: $!\n";
    }
}

sub extract_dataset {
    print "Dataset: $ARGV[0]\n";
    &extract_base();
    &extract_timestamp("deletedvalidity");
    &extract_list("entries", \&extract_entry);
    &extract_list("indicies", \&extract_index);
}
