/* kerberos.cc - kerberos authentication module
 * Rob Earhart
 * $Id: kerberos.cc,v 1.4 1998/03/09 00:58:30 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include <unistd.h>
extern "C" {
#include <krb.h>
#include <des.h>
}
#include "response.h"
#include "auth.h"
#include "init.h"
#include "mutex.h"
#include "grab.h"
#include "conn.h"
#include "hname.h"

static Mutex des_mutex;		// like I trust des_random_key()
				// to be reentrant... not.

static void AuthKrb(Conn *conn) {
  union {
    des_cblock key;
    u_int32_t checksum;
    unsigned char str[sizeof(des_cblock)];
  } challenge;

  conn->GetCRLF();

  {
    Grab g(des_mutex);
    des_random_key(challenge.key);
  }

  challenge.checksum = htonl(challenge.checksum);
  {
    Grab g(conn->OutputMutex());
    (conn->Out() << "+ {4}\r\n").write(challenge.str, 4) << "\r\n" << flush;
  }

  String response = conn->GetNextString();
  conn->GetCRLF();

  KTEXT_ST kt;
  kt.length = response.size();
  copy(response.begin(),
       min(response.end(), response.begin() + MAX_KTXT_LEN),
       kt.dat);
  kt.mbz = 0;
  AUTH_DAT auth_data;
  if (krb_rd_req(&kt, "acap", (char *)Hostname::Instance()->Name(),
		 conn->GetPeerAddr().sin_addr.s_addr,
		 &auth_data, "")
      || auth_data.checksum != challenge.checksum)
    throw Response::NO("Kerberos_V4 authentication failed");
  challenge.checksum = htonl(ntohl(challenge.checksum) + 1);
  challenge.key[4] = 1;		// no security layer yet
  challenge.key[5] =
    challenge.key[6] =
    challenge.key[7] = 0;	// ciphertext buffersize = 0
  des_key_schedule schedule;
  des_key_sched(auth_data.session, schedule);
  des_ecb_encrypt(challenge.str, challenge.str, schedule, DES_ENCRYPT);
  {
    Grab g(conn->OutputMutex());
    (conn->Out() << "+ {8}\r\n").write(challenge.str, 8) << "\r\n" << flush;
  }
  response = conn->GetNextString();
  if (response.size() < 16 || response.size() % 8)
    throw Response::NO("Kerberos_V4 authentication failed");
  des_pcbc_encrypt((unsigned char *)response.octets(),
		   (unsigned char *)response.octets(),
		   response.size(), schedule,
		   auth_data.session, DES_DECRYPT);
  if (((u_int32_t *)response.octets())[0]
      != auth_data.checksum
      || response[4] != 1)
    throw Response::NO("Kerberos_V4 authentication failed");

  conn->GetCRLF();

  DEBUG("Kerberos_v4 login as " << response.c_str() + 8);
  conn->SetAuth(String(response.c_str() + 8));
}

static void SetupKerberos() {
  Auth::Mechs()["kerberos_v4"] = &AuthKrb;
}

static Init init(&SetupKerberos);
