/* modifiers.cc - modifiers implementation
 * Rob Earhart
 * $Id: modifiers.cc,v 1.7 1998/03/09 00:58:33 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include "modifiers.h"
#include "response.h"
#include "context.h"

Modifiers::Modifiers()
  : gotenumerate(false), gotnotify(false),
    gotmakecontext(false), gotreturn(false),
    gotsort(false), gotdepth(false),
    gothardlimit(false), gotnoinherit(false),
    depth(1) {}

void Modifiers::SetEnumerate() {
  if (gotenumerate)
    throw Response::BAD("Enumerate specified multiple times");
  gotenumerate = true;
}

void Modifiers::SetNotify() {
  if (gotnotify)
    throw Response::BAD("Notify specified multiple times");
  gotnotify = true;
}

void Modifiers::SetMakeContext(const String &context) {
  if (gotmakecontext)
    throw Response::BAD("Makecontext specified multiple times");
  Context::CheckLegalContextName(context);
  gotmakecontext = true;
  makecontext = context;
}

void Modifiers::SettingReturnList() {
  if (gotreturn)
    throw Response::BAD("Return list specified multiple times");
  gotreturn = true;
}

void Modifiers::Metadata::AddMetadatum(const String &data) {
  // XXX This probably ought to be paramaterized somehow
  // so that we can dynamically plug in new metadata types...
  Token::token tok = Token::Lookup(data);
  switch (tok) {
  case Token::A_ATTRIBUTE:
  case Token::A_MYRIGHTS:
  case Token::A_SIZE:
  case Token::A_VALUE:
  case Token::A_ACL:
  case Token::A_CONTENTTYPE:
    metadata.push_front(tok);
    break;
  default:
    throw Response::BAD("Unknown metadata type");
  }
}

void Modifiers::Metadata::UseDefaultMetadata(const String &attribute) {
  if (1 <= attribute.size()
      && attribute[attribute.size() - 1] == '*')
    metadata.push_front(Token::A_ATTRIBUTE);
  metadata.push_front(Token::A_VALUE);
}

void Modifiers::AddReturn(const String &attribute, Ptr<Metadata> &metadata) {
  metadata->Reverse();
  returnlist.push_front(returnlist_t::value_type(attribute, metadata));
}

void Modifiers::DoneWithReturnList() {
  returnlist.reverse();
}

void Modifiers::SettingSortList() {
  if (gotsort)
    throw Response::BAD("Sort list specified multiple times");
  gotsort = true;
}

void Modifiers::AddSort(const String &attribute, const String &comparator) {
  sortlist.push_front(sortlist_t::value_type(attribute,
					     Compare::Instance()->LookupOrdering(comparator)));
}

void Modifiers::DoneWithSortList() {
  sortlist.reverse();
}

void Modifiers::SetDepth(unsigned long d) {
  if (gotdepth)
    throw Response::BAD("Search depth specified multiple times");
  gotdepth = true;
  depth = d;
}

void Modifiers::SetHardLimit(unsigned long l) {
  if (gothardlimit)
    throw Response::BAD("Search hardlimit specified multiple times");
  if (l == 0)
    throw Response::BAD("Hardlimit requires a non-zero limit");
  gothardlimit = true;
  hardlimit = l;
}

void Modifiers::SetLimit(unsigned long l, unsigned long m) {
  if (gotlimit)
    throw Response::BAD("Search limit specified multiple times");
  gotlimit = true;
  limit = l;
  maxlimit = m;
}

void Modifiers::SetNoInherit() {
  if (gotnoinherit)
    throw Response::BAD("NoInherit specified multiple times");
  gotnoinherit = true;
}

