/* trx.h -*- C++ -*- ACAP transaction interface
 * Rob Earhart
 * $Id: trx.h,v 1.2 1998/03/09 00:58:58 rob Exp $
 */
/***********************************************************
        Copyright 1998 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_TRX_H
#define ACAP_TRX_H

#include <strstream.h>
#include <hash_set.h>
#include "timestamp.h"
#include "void.h"

class PS;
class PSResources;

class Trx {
public:
  static void Recover();	// called at startup time to recover pending transactions
				// completes when all recovery is done
				// This may issue tasks, so it'd be a REALLY GOOD IDEA
				// if the task processing threads have been started.
  Trx(Timestamp *t);		// create a new transaction
  void Commit();		// synchronizes a transaction to disk
				// result is OK once Commit() returns.
  ~Trx();			// note - this MAY take a little while,
				// since it applies the transaction...

  Timestamp::Stamp TrxStamp() const { return tstamp->GetStamp(); }
  const char *TrxTimeStr() const { return tstamp->LongStr(); }

  // Journaled operations
  void SetPS(PS *ps) {
    if (ps != current_ps) {
      current_ps = ps;
      WriteSetPS(ps);
    }
  }
  void WriteGrowTo(size_t size) {
    tfile << 'g' << size << '\0';
  }
  void WriteOffset(off_t loc, off_t data) {
    tfile << 'o' << loc << '\0' << data << '\0';
  }
  void WriteData(off_t loc, void *data, size_t len) {
    (tfile << 'd' << loc << '\0' << len << '\0').write((char *)data, (int)len) << '\0';
  }
  void WriteAlloc(off_t loc, size_t len) {
    tfile << 'a' << loc << '\0' << len << '\0';
  }
  void WriteFree(off_t loc, size_t size) {
    tfile << 'f' << loc << '\0' << size << '\0';
  }
  void AddPSBlob(PSBlob *blob) {
    tstamp->AddPSBlob(blob);
  }

private:
  PS *current_ps;
  void WriteSetPS(PS *ps);
  static void Apply(istream &trx);
  bool committed;
  Timestamp *tstamp;
  String tname;
  strstream tfile;
  typedef hash_set<P<PSResources>, hash<void *>, equal_to<void *> > pset_t;
  pset_t pset;
};

#endif // ACAP_TRX_H
