/* util.cc - utility implementation
 * Rob Earhart
 * $Id: util.cc,v 1.1.1.1 1997/11/26 21:01:10 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#include <unistd.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "util.h"
#include "unixerror.h"

Align align;

static char hexchar[16] = {
  '0', '1', '2', '3', '4', '5', '6', '7',
  '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'
};

void FD::Close() {
  int cfd = fd;
  fd = -1;
  if (0 <= cfd
      && close(cfd))
    throw UnixError("Closing a file");
}

void FD::Open(const char *name, int flags, int mode) {
  Close();
  fd = open(name, flags, mode);
  if (fd < 0)
    throw UnixError(name);
}

void MMap::MapIn(caddr_t _addr, size_t _len, int prot,
		 int flags, int fd, off_t off) {
  MapOut();
  len = _len;
  addr = mmap(_addr, len, prot, flags, fd, off);
  if (addr == (caddr_t)-1)
    throw UnixError("Establishing a memory mapping");
}

void MMap::MapOut() {
  if (addr != (caddr_t)-1) munmap(addr, len);
  addr = (caddr_t)-1;
}
