/* util.h -*- C++ -*- utility interface
 * Rob Earhart
 * $Id: util.h,v 1.2 1998/02/18 21:20:25 rob Exp $
 */
/***********************************************************
        Copyright 1997 by Carnegie Mellon University

                      All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of CMU not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
******************************************************************/

#ifndef ACAP_UTIL_H
#define ACAP_UTIL_H

#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include "str.h"

struct Align {
  union {
    long w;
    void *p;
  } data;
  Align() { data.w = 0; data.p = 0; }
};

extern Align align;

// These provide automatic resource cleanup
class FD {
public:
  FD(int _fd = -1) : fd(_fd) {}
  FD(const char *name, int flags, int mode = S_IRUSR|S_IWUSR) : fd(-1) {
    Open(name, flags, mode);
  }
  ~FD() { if (fd >= 0) close(fd); }
  operator int() { return fd; }
  void Open(const char *name, int flags, int mode = S_IRUSR|S_IWUSR);
  void Close();
private:
  int fd;
};

class MMap {
public:
  // Constructs a Null map
  MMap() : addr((caddr_t)-1), len(0) {}

  // Constructs a new memory mapping
  MMap(caddr_t _addr, size_t _len, int prot,
       int flags, int fd, off_t off)
    : addr((caddr_t)-1), len(_len) {
      MapIn(_addr, _len, prot, flags, fd, off);
  }

  // Establishes a new memory mapping for an existing object
  void MapIn(caddr_t _addr, size_t _len, int prot,
	     int flags, int fd, off_t off);

  // Unmaps the current object
  void MapOut();

  ~MMap() { MapOut(); }

  operator char *() { return (char *)addr; }

  char *Map() { return (char *)addr; }

  size_t Length() const { return len; }

private:
  caddr_t addr;
  size_t len;
};

#endif // ACAP_UTIL_H
